var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Functions_native, _Functions_app;
import { HttpsErrorCode, } from "./common";
import { deserialize, firebase, FirebaseApp, serialize } from "@nativescript/firebase-core";
let defaultFunctions;
const fb = firebase();
Object.defineProperty(fb, 'functions', {
    value: (app) => {
        if (!app) {
            if (!defaultFunctions) {
                defaultFunctions = new Functions();
            }
            return defaultFunctions;
        }
        return new Functions(app);
    },
    writable: false,
});
function errorToCode(error) {
    let code = HttpsErrorCode.UNKNOWN;
    switch (error) {
        case com.google.firebase.functions.FirebaseFunctionsException.Code.OK:
            code = HttpsErrorCode.OK;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.CANCELLED:
            code = HttpsErrorCode.CANCELLED;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.UNKNOWN:
            code = HttpsErrorCode.UNKNOWN;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.INVALID_ARGUMENT:
            code = HttpsErrorCode.INVALID_ARGUMENT;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.DEADLINE_EXCEEDED:
            code = HttpsErrorCode.DEADLINE_EXCEEDED;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.NOT_FOUND:
            code = HttpsErrorCode.NOT_FOUND;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.ALREADY_EXISTS:
            code = HttpsErrorCode.ALREADY_EXISTS;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.PERMISSION_DENIED:
            code = HttpsErrorCode.PERMISSION_DENIED;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.RESOURCE_EXHAUSTED:
            code = HttpsErrorCode.RESOURCE_EXHAUSTED;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.FAILED_PRECONDITION:
            code = HttpsErrorCode.FAILED_PRECONDITION;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.ABORTED:
            code = HttpsErrorCode.ABORTED;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.OUT_OF_RANGE:
            code = HttpsErrorCode.OUT_OF_RANGE;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.UNIMPLEMENTED:
            code = HttpsErrorCode.UNIMPLEMENTED;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.INTERNAL:
            code = HttpsErrorCode.INTERNAL;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.UNAVAILABLE:
            code = HttpsErrorCode.UNAVAILABLE;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.DATA_LOSS:
            code = HttpsErrorCode.DATA_LOSS;
            break;
        case com.google.firebase.functions.FirebaseFunctionsException.Code.UNAUTHENTICATED:
            code = HttpsErrorCode.UNAUTHENTICATED;
            break;
        default:
            break;
    }
    return code;
}
export class HttpsError extends Error {
    constructor(code, message, details = null, native = null) {
        super(message);
        this.code = code;
        this.details = details;
        this.native = native;
    }
}
function toHttpsError(error) {
    let details = null;
    let message = error.getMessage?.();
    let code = HttpsErrorCode.UNKNOWN;
    const cause = error.getCause();
    if (cause instanceof com.google.firebase.functions.FirebaseFunctionsException) {
        const functionsException = cause;
        details = functionsException.getDetails();
        code = errorToCode(functionsException.getCode());
        message = functionsException.getMessage?.();
        const isTimeout = code === HttpsErrorCode.DEADLINE_EXCEEDED;
        if (functionsException.getCause?.() instanceof java.io.IOException && !isTimeout) {
            code = HttpsErrorCode.UNAVAILABLE;
            message = HttpsErrorCode.UNAVAILABLE;
        }
    }
    return new HttpsError(code, message, details, error);
}
export class Functions {
    constructor(app) {
        _Functions_native.set(this, void 0);
        _Functions_app.set(this, void 0);
        if (app?.native) {
            __classPrivateFieldSet(this, _Functions_native, com.google.firebase.functions.FirebaseFunctions.getInstance(app.native), "f");
        }
        else {
            if (defaultFunctions) {
                return defaultFunctions;
            }
            defaultFunctions = this;
            __classPrivateFieldSet(this, _Functions_native, com.google.firebase.functions.FirebaseFunctions.getInstance(), "f");
        }
    }
    httpsCallable(name, options) {
        const callable = this.native.getHttpsCallable(name);
        if (typeof options?.timeout === 'number') {
            callable.setTimeout(options.timeout, java.util.concurrent.TimeUnit.SECONDS);
        }
        return (data) => {
            return new Promise((resolve, reject) => {
                org.nativescript.firebase.functions.FirebaseFunctions.call(callable, data ? serialize(data) : null, new org.nativescript.firebase.functions.FirebaseFunctions.Callback({
                    onSuccess(result) {
                        resolve(deserialize(result?.getData?.()));
                    }, onError(error) {
                        reject(toHttpsError(error));
                    }
                }));
            });
        };
    }
    useEmulator(host, port) {
        this.native.useEmulator(host, port);
    }
    get native() {
        return __classPrivateFieldGet(this, _Functions_native, "f");
    }
    get ios() {
        return this.native;
    }
    useFunctionsEmulatorOrigin(origin) {
        this.native.useFunctionsEmulator(origin);
    }
    get app() {
        if (!__classPrivateFieldGet(this, _Functions_app, "f")) {
            // @ts-ignore
            __classPrivateFieldSet(this, _Functions_app, FirebaseApp.fromNative(this.native.getApp()), "f");
        }
        return __classPrivateFieldGet(this, _Functions_app, "f");
    }
}
_Functions_native = new WeakMap(), _Functions_app = new WeakMap();
//# sourceMappingURL=index.android.js.map